<?php
/**
 * fsl2006 congress
 * topic: PHP router in Asterisk
 * author: Moises Silva <moy@ivsol.net>
 * @package fsl-astrouter
 */

/**
 * AgiConnector class. Quick and clean communication
 * with AGI
 */
class AgiConnector
{
	/**
	 * container for all the initial
	 * AGI variables sent by Asterisk
	 * @var object ArrayObject
	 */
	private $agi_variables   = NULL;

	/**
	 * array struct of a typical
	 * AGI command response
	 * @var array
	 */
	private $response_struct = array
	(
		'result'             => 0, /* result from command              */
		'data'               => '' /* string, possible associated data */
	);

	/**
	 * POSIX regular expression
	 * to match the AGI commands responses
	 * @var string
	 */
	private $success_regexp  = '200[[:space:]]+result=([0-9]+)?([[:space:]]+\(.*\))?';
	
	/**
	 * instance the ArrayObject to contain the agi variables
	 * and read the agi variables sent by Asterisk from STDIN
	 * @param void
	 */
	public function __construct()
	{
		$this->agi_variables = new ArrayObject();
		$this->ReadAgiVariables();
	}

	/**
	 * quick and dirty log
	 * @return void
	 * @param string $Message
	 */
	private function LogMessage($Message)
	{
		file_put_contents('/tmp/agi.log', $Message . "\n", FILE_APPEND);
	}
	
	/**
	 * read the initial AGI variables and save them
	 * in $this->agi_variables
	 * @return void
	 * @param void
	 */
	public function ReadAgiVariables()
	{
		do
		{
			$this->LogMessage('reading environment');
			$read_string = $this->ReadFromAsterisk();
			$read_lines  = explode("\n", $read_string);
			foreach ( $read_lines as $read_line )
			{
				$this->LogMessage('reading line: ' . $read_line);
				if ( FALSE !== ( $dot_position = strpos($read_line, ':') ) )
				{
					list($parameter_name, $parameter_value) = explode(':', $read_line, 2);
					$parameter_value                        = trim($parameter_value);
					$this->agi_variables->offsetSet($parameter_name, $parameter_value);
					$this->LogMessage('setting variable ' . $parameter_name . ' to ' . $parameter_value);
				}
			}	
		} while ( !$this->agi_variables->offsetExists('agi_accountcode') );
	}

	/**
	 * read from Asterisk (STDIN) 
	 * and log what is read
	 * @return void
	 * @param void
	 */
	private function ReadFromAsterisk()
	{
		$this->LogMessage('reading from Asterisk');
		$read = fread(STDIN, 1024);
		$this->LogMessage("read: {$read}");
		return $read;
	}

	/**
	 * write to Asterisk (STDOUT)
	 * and log what is wrote
	 * @return int
	 * @param string $CommandString
	 */ 
	private function WriteToAsterisk($CommandString)
	{
		$this->LogMessage('writing to Asterisk: ' . $CommandString);
		$wrote = fwrite(STDOUT, $CommandString . "\n");
		$this->LogMessage("wrote: {$wrote}");
		return $wrote;
	}

	/**
	 * read the string response sent by Asterisk
	 * and parse it into response_struct
	 * @return struct asterisk_response
	 * @param void
	 */
	private function ReadAsteriskResponse()
	{
		$this->LogMessage('reading Asterisk command response');
		$response                   = $this->ReadFromAsterisk();
		$asterisk_response          = $this->response_struct;
		if ( FALSE !== eregi($this->success_regexp, $response, $matches) )
		{
			$asterisk_response['result'] = $matches[1];
			$asterisk_response['data']   = trim(str_replace(array('(',')'), "", $matches[2]));
		}
		$this->LogMessage('result=' . $asterisk_response['result']);
		$this->LogMessage('data='. $asterisk_response['data']);
		return $asterisk_response;
	}

	/**
	 * @return struct asterisk_response
	 * @param string $CommandString
	 */
	public function ExecuteCommand($CommandString)
	{
		$this->WriteToAsterisk($CommandString);
		return $this->ReadAsteriskResponse();
	}

	/**
	 * simply ejecute the given application with the proper arguments
	 * @return void
	 * @param string $ApplicationName
	 * @param string $Arguments
	 */
	public function ExecuteApplication($ApplicationName, $Arguments)
	{
		$command = "EXEC {$ApplicationName} {$Arguments}";
		$this->ExecuteCommand($command);
	}

	/**
	 * send verbose messages to Asterisk
	 * @return void
	 * @param string $Message
	 * @param int $VerbosityLevel
	 */
	public function Write($Message, $VerbosityLevel = 1)
	{
		$message = (string)$Message;
		$message = str_replace("\\", "\\\\", $message);
		$message = str_replace("\"", "\\\"", $message);
		$message = str_replace("\n", "\\n", $message);
		$command = "VERBOSE \"{$message}\" {$VerbosityLevel}";
		$this->ExecuteCommand($command);
	}

	/**
	 * get specified chanel variable value from
	 * Asterisk
	 * @return string
	 * @param string $VariableName
	 */
	public function GetVariable($VariableName)
	{
		$command           = "GET VARIABLE {$VariableName}";
		$asterisk_response = $this->ExecuteCommand($command);
		return $asterisk_response['data'];
	}

	/**
	 * set specified channel variable in
	 * Asterisk
	 * @return void
	 * @param string $VariableName
	 * @param string $VariableValue
	 */
	public function SetVariable($VariableName, $VariableValue)
	{
		$command = "SET VARIABLE {$VariableName} \"{$VariableValue}\"";
		$this->ExecuteCommand($command);
	}

	/**
	 * var_dump displayed in Asterisk Console
	 * @return void
	 * @param mixed $Variable
	 */
	public function DumpVariable($Variable)
	{
		ob_start();
		var_dump($Variable);
		$variable_dump = ob_get_contents();
		ob_end_clean();
		$this->Write("===== DUMPING VARIABLE ====");
		$this->Write($variable_dump);
	}

	/**
	 * get the specified AGI variable
	 * (AGI variables are read only once when the script begins)
	 * @return string
	 * @param string $VariableName
	 */
	public function GetAgiVariable($VariableName)
	{
		$variable = 'agi_' . $VariableName;
		if ( $this->agi_variables->offsetExists($variable) )
		{
			return $this->agi_variables->offsetGet($variable);
		}
		return NULL;
	}

	/**
	 * wrapper for Playback Asterisk Application
	 * @return void
	 * @param string $FileName
	 */
	public function Playback($FileName)
	{
		$this->ExecuteApplication('Playback', $FileName);
	}

	/**
	 * wrapper for SayDigits Asterisk Application
	 * @return void
	 * @param string $Digits
	 */
	public function SayDigits($Digits)
	{
		$this->ExecuteApplication('SayDigits', $Digits);
	}

	/**
	 * wrapper for Dial Asterisk Application
	 * @return void
	 * @param string $DialString
	 */
	public function Dial($DialString)
	{
		$this->ExecuteApplication('Dial', $DialString);
	}

	/**
	 * wrapper for MeetMe Asterisk Application
	 * @return void
	 * @param int $ConferenceNumber
	 */
	public function StartConference($ConferenceNumber)
	{
		$this->ExecuteApplication('MeetMe', $ConferenceNumber);
	}

	/**
	 * wrapper for VoicemailMain Asterisk Application
	 * @return void
	 * @param int $VoiceboxNumber
	 */
	public function EnterVoicemail($VoiceboxNumber)
	{
		$this->ExecuteApplication('VoicemailMain', $VoiceboxNumber);
	}

	/**
	 * wrapper for GET DATA command
	 * prompt the user for digits using $PlayFile
	 * and wait $Timeout milliseconds restarting the timer
	 * each time a DTMF digit is received. A maximum of digits
	 * may be specified as third parameter
	 * @return int
	 * @param string $PlayFile
	 * @param int $Timeout
	 * @param int $MaxDigits
	 */
	public function GetDigitsFromUser($PlayFile, $Timeout = 2000, $MaxDigits = NULL)
	{
		if ( NULL !== $MaxDigits )
		{
			$response = $this->ExecuteCommand("GET DATA \"{$PlayFile}\" \"{$Timeout}\" \"{$MaxDigits}\"");
		}
		else
		{
			$response = $this->ExecuteCommand("GET DATA \"{$PlayFile}\" \"{$Timeout}\"");
		}
		return $response['result'];
	}
	
}
?>
