<?php
/**
 * fsl2006 congress
 * topic: PHP router in Asterisk
 * author: Moises Silva <moy@ivsol.net>
 * @package fsl-astrouter
 */

/**
 * RouteHoncho class. Retrieve data from database and
 * match dialed numbers against system dial patterns
 */
class RouteHoncho
{
	/**
	 * application identifiers
	 */
	const APPLICATION_DIAL       = 'dial';
	const APPLICATION_VOICEMAIL  = 'voicemail';
	const APPLICATION_CONFERENCE = 'conference';

	/**
	 * sqlite database Data Source Name (DSN)
	 */
	const DATABASE_DSN           = 'sqlite:/tmp/route.db';

	/**
	 * array of mappings between pattern => application info
	 * @var array
	 */
	private $applications_map    = array();

	/**
	 * connection to database using PDO object
	 * @var object PDO
	 */
	private $database = NULL;

	/**
	 * instance the PDO opening the connection
	 * to the database. Set Exception error handling in PDO and
	 * finally save in memory all the database mappings 
	 * @param void
	 */
	public function __construct()
	{
		$this->database  = new PDO(self::DATABASE_DSN);
		$this->database->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
		foreach ( $this->database->query('SELECT * FROM applications_mappings', PDO::FETCH_OBJ) as $row )
		{
			$this->applications_map[$row->pattern]['name']    = $row->application_name;
			$this->applications_map[$row->pattern]['data']    = $row->application_data;
			$this->applications_map[$row->pattern]['pattern'] = $row->pattern;
			$this->applications_map[$row->pattern]['map_id']  = $row->map_id;
			$this->applications_map[$row->pattern]['tokens']  = $this->GetPatternTokens($row->pattern);
		}
	}

	/**
	 * quick and dirty log method
	 * @return void
	 * @param string $Message
	 */
	public function LogMessage($Message)
	{
		file_put_contents('/tmp/honcho.log', FILE_APPEND);
	}

	/**
	 * this method authorize call requests.
	 * If a $UserPin is provided, will attempt to authorize the
	 * call using the user permissions (the owner of the PIN)
	 * otherwise will use the $ExtensionNumber to find out
	 * if the call proceeds
	 * @return bool
	 * @param int $ExtensionNumber
	 * @param int $MapId
	 * @param int $UserPin
	 */
	public function CallIsAuthorized($ExtensionNumber, $MapId, $UserPin = NULL)
	{
		if ( NULL == $UserPin )
		{
			$sql  = 'SELECT ext_map_id FROM extensions_mappings INNER JOIN extensions ON ';
			$sql .= 'extensions_mappings.extension_id = extensions.extension_id INNER JOIN applications_mappings ON ';
			$sql .= 'extensions_mappings.map_id       = applications_mappings.map_id WHERE ';
			$sql .= 'extensions.extension_number = :extension_number AND applications_mappings.map_id = :map_id';
			$stmt = $this->database->prepare($sql);
			$this->LogMessage($sql);
			$this->LogMessage($ExtensionNumber);
			$this->LogMessage($MapId);
			$stmt->execute(array(':extension_number' => $ExtensionNumber, ':map_id' => $MapId));
		}
		else
		{
			$sql  = 'SELECT usr_map_id FROM users_mappings INNER JOIN users ON ';
			$sql .= 'users_mappings.user_id = users.user_id INNER JOIN applications_mappings ON ';
			$sql .= 'users_mappings.map_id = applications_mappings.map_id WHERE ';
			$sql .= 'users.user_pin = :user_pin AND applications_mappings.map_id = :map_id';
			$stmt = $this->database->prepare($sql);
			$this->LogMessage($sql);
			$this->LogMessage($UserPin);
			$this->LogMessage($MapId);
			$stmt->execute(array(':user_pin' => $UserPin, ':map_id' => $MapId));
		}
		if ( FALSE !== $stmt->fetch() )
		{
			return TRUE;
		}
		return FALSE;
	}

	/**
	 * return the map_struct corresponding
	 * to the dialed number or NULL if the number
	 * does not match any pattern
	 * @return int
	 * @param int $DialedNumber
	 */
	public function GetNumberMapping($DialedNumber)
	{
		$DialedNumber = (string)$DialedNumber;
		/* comparing each number agains the tokens */
		$number_length     = strlen($DialedNumber);
		$possible_patterns = array();
		/* first discard all non matching length patterns */
		foreach ( $this->applications_map as $pattern => $application_map ) 
		{
			if ( ( count($application_map['tokens']) == $number_length       ) || /* match the number in tokens OR. */
			     ( ( count($application_map['tokens']) < $number_length ) &&
				   ( strpos($pattern, '.') !== FALSE                    )    )    /* less tokens but the pattern has a dot */
			   )
			{
				$possible_patterns[$pattern] = $application_map;
			}   
		}
		if ( empty($possible_patterns) )
		{
			return NULL;
		}
		/* now see which is the first pattern that matches all the tokens (ignoring specificity of the patterns) */
		$current_position = 0;
		while ( $current_position < $number_length )
		{
			$matching_patterns = array();
			$current_number    = substr($DialedNumber, $current_position, 1);
			foreach ( $possible_patterns as $pattern => $application_map )
			{
				if ( in_array($current_number, $application_map['tokens'][$current_position]) )
				{
					$matching_patterns[$pattern] = $application_map;
				}
			}
			if ( empty($matching_patterns) )
			{
				return NULL;
			}
			$possible_patterns = $matching_patterns;
			$current_position++;
		}
		return current($possible_patterns); /* return the first match */
	}

	/**
	 * create the tokens structure
	 * using $Pattern as basis
	 * @return array
	 * @param string $Pattern
	 * @throw InvalidArgumentException (when the pattern is not valid)
	 */
	private function GetPatternTokens($Pattern)
	{
		$tokens             = array();
		$character_position = 0;
		$number_length      = strlen($Pattern);
		while ( $character_position < $number_length )
		{
			$current_character = strtoupper(substr($Pattern, $character_position, 1));
			if ( is_numeric($current_character) )
			{
				$tokens[] = array($current_character);
				$character_position++;
			}
			elseif ( 'X' == $current_character || '.' == $current_character )
			{
				$tokens[] = range(0,9);
				$character_position++;
				if ( '.' == $current_character ) /* ignore anything after the dot */
				{
					break;
				}
			}
			elseif ( 'N' == $current_character )
			{
				$tokens[] = range(0,9);
				$character_position++;
			}
			elseif ( '[' == $current_character )
			{
				/* make sure we have a closing bracket */
				if ( FALSE === ( $closing_bracket_position = strpos($Pattern, ']', $character_position) ) )
				{
					throw new InvalidArgumentException('hu? pattern has has open bracket but no closing bracket?');
				}
				$brackets_chars      = substr($Pattern, $character_position + 1, $closing_bracket_position - 1);
				$character_position  = $closing_bracket_position + 1;
				/* make sure we have a valid bracket range or literal numbers */
				if ( FALSE === ereg('^[0-9]+|[0-9]-[0-9]$', $brackets_chars) )
				{
					throw new InvalidArgumentException('bracket string inside pattern not valid!');
				}
				if ( FALSE !== strpos($brackets_chars, '-') )
				{
					list($low_range, $high_range) = explode('-', $brackets_chars);
					$tokens[] = range($low_range, $high_range);
				}
				else
				{
					$next_token_number          = count($tokens);
					$tokens[$next_token_number] = array();
					$numbers_length             = strlen($brackets_chars);
					for ( $i = 0; $i < $numbers_length; $i++ )
					{
						$tokens[$next_token_number][] = substr($brackets_chars, $i, 1);
					}
				}
			}
			else
			{
				throw InvalidArgumentException('cannot convert pattern to tokens, invalid pattern received!');
			}
		}
		return $tokens;
	}
}
?>
