<?php
/**
 * require needed external libraries
 * 
 */
require_once 'netsock/SocketClient.php';
require_once 'stdlog/Logger.php';
require_once 'FailedManagerLoginException.php';

/**
 * AmiConnector
 * This class has methods to manage the connection with
 * the Asterisk Manager Interface through the localhost:5038 socket
 * depends on the Logger class, needs to be included by the implementator
 */
class AmiConnector extends SocketClient
{
	/**
	 * resource string that tell us protocol, server and other usefull stuff
	 * @var resource
	 * @access protected
	 */
	protected $socket_uri       = "tcp://localhost:5038";
	
	/**
	 * username to connect to asterisk manager
	 * @var string
	 * @access private
	 */
	private   $user              = '';
	
	/**
	 * password to connect to asterisk manager
	 * @var string
	 * @access private
	 */
	private   $password       = '';
	
	/**
	 * stores the unified data received from Asterisk Manager
	 * @var array
	 * @access private
	 */
	private $manager_data     = "";
	
	/**
	 * stores commands to be sent on CommitCommands()
	 * @var array
	 * @access private
	 */
	private $commands_queue   = array();
	
	/**
	 * Asterisk Call Manager server welcome message
	 * @const
	 */
	const MANAGER_WELCOME     = 'Asterisk Call Manager';
	
	/**
	 * this chars are used to separate each block of events/responses/actions
	 * @const
	 */
	const MANAGER_END_CHARS   = "\r\n\r\n";
	
	/**
	 * this chars are used to separate the arguments of each event/response/action
	 * @const
	 */
	const MANAGER_ARG_DIV     = "\r\n";
	
	/**
	 * debugging escape characters
	 * @const
	 */
	const DEBUG_ESCAPE        = '\r\n';
	
	/**
	 * set the parameters to connect to manager
	 * @param string $User
	 * @param string $Password
	 */
	public function SetAuthenticationInfo($User, $Password)
	{
		$this->user     = $User;
		$this->password = $Password;
	}
	
	/**
	 * connects to the manager
	 * @param String $SocketUri
	*/
	public function ConnectToManager($SocketUri = NULL)
	{
		$this->socket_uri = NULL === $SocketUri ? $this->socket_uri : $SocketUri;
		$this->SocketConnect($this->socket_uri);
	}
	
	/**
	 * implementation of the parent abstract method OnConnect
	 * @param void
	 * @return void
	 * @access protected
	 */
	protected function OnConnect()
	{
		/* Reading manager output buffer on connection */
		$connect_string = $this->SocketRead();
		/* Verifying output */
		if ( strstr($connect_string, self::MANAGER_WELCOME) )
		{
			Logger::WriteLog("Connected to Asterisk Manager succesfully.");
			Logger::WriteLog("Trying to login...");
		}
		else
		{
			Logger::WriteLog("Hu??? no Asterisk welcome message?.", Logger::LOG_LEVEL_WARNING);
		}
		$this->LogIn();
	}
	
	/**
	 * implementation of the parent abstract method OnDisconnect
	 * @param void
	 * @return void
	 * @access protected
	 */
	protected function OnDisConnect()
	{
		Logger::WriteLog("Disconnected from Asterisk Manager succesfully.");
	}
	
	/**
	 * implementation of the parent abstract method OnReconnect
	 * @param void
	 * @return void
	 * @access protected
	 */
	protected function OnReConnect()
	{
		Logger::WriteLog("Reconnected to Asterisk Manager succesfully.");
	}

	/**
	 * try to login to the manager server
	 * @param void
	 * @return void 
	 * @access private
	 */
	private function LogIn()
	{
		$login_action  = array
		(
			'Action'   => 'Login',
			'Username' => $this->user,
			'Secret'   => $this->password
		);
		$this->SendDataToManager($login_action);
		$login_response = $this->ForceReceiveManagerPackets();
		$date = date(AmiConnector::$date_format);
		if ( strtolower($login_response['Response']) == 'success' )
		{
			Logger::WriteLog("Log in to Asterisk Manager succesfull on {$date}");
		}
		else
		{
			$this->SocketClose();
			Logger::WriteLog("Unsuccesfull Asterisk Manager login attempt on {$date}");
			throw new FailedManagerLoginException('failed Asterisk Manager login');
		}
	}
	
	/**
	 * send data to the manager, that is, write to the manager socket
	 * @param array $ManagerDataArray
	 * @return void
	 */
	public function SendDataToManager(array $ManagerDataArray)
	{
		$manager_data = '';
		/* separate each argument by the designed chars */
		foreach ( $ManagerDataArray as $data_key => $data_value )
		{
			$manager_data .= "{$data_key}: {$data_value}" . AmiConnector::MANAGER_ARG_DIV;
		}
		/* strip the last separator since the last argument does not need to be separated */
		$manager_data  = substr($manager_data, 0, strlen($manager_data) - strlen(AmiConnector::MANAGER_ARG_DIV));
		/* append the ending chars for this data */
		$manager_data .= AmiConnector::MANAGER_END_CHARS;
		$this->SocketWrite($manager_data);
	}
	
	/**
	 * read the manager socket info, and return the received
	 * manager packets. A single reading it may return less data
	 * than a complete manager packet, or more data of a whole packet
	 * this function should return an array with full packets, or an empty array
	 * in case that no fullpacket has been received yet
	 * @param void
	 * @return array
	 */
	public function ReceiveManagerPackets()
	{
		static $manager_buffer     = "";      /* by default, the packet_buffer is empty */
		$manager_packets           = array(); /* complete manager packets, 0 by default */
		$parsed_manager_packets    = array(); /* complete and parsed manager packets, 0 by default */
		
		/* then complete the manager buffer with the socket reading  */
		$manager_reading = $this->SocketRead();
		$manager_buffer .= $manager_reading;
		
		/* display debugging info */
		$manager_read = str_replace(AmiConnector::MANAGER_ARG_DIV, AmiConnector::DEBUG_ESCAPE, $manager_reading);
		Logger::WriteLog("read from manager socket {$manager_read}", Logger::LOG_LEVEL_DEBUG);
		
		/* if some packet(s) is(are) ending in the manager buffer */
		if ( strpos($manager_buffer, AmiConnector::MANAGER_END_CHARS) !== FALSE )
		{
			/* separe the packet chunks to join the whole packets */
			$packets_chunks  = explode(AmiConnector::MANAGER_END_CHARS, $manager_buffer);
			
			/* the last element will be saved to the buffer since is empty, or not complete */
			$manager_buffer = array_pop($packets_chunks);
			while ( ( $manager_packet = array_shift($packets_chunks) ) )
			{
				$manager_packets[] = $manager_packet;
			}
		}
		else /* otherwise just notify about incomplete packet */
		{
			$debug_message = "unable to retrieve complete packet, current manager buffer is: {$manager_buffer}";
			Logger::WriteLog($debug_message, Logger::LOG_LEVEL_DEBUG);
		}
		foreach ( $manager_packets as $manager_packet )
		{
			$parsed_manager_packets[] = AmiConnector::ParseManagerPacket($manager_packet);
		}
		return $parsed_manager_packets;
	}
	
	/**
	 * it will keep trying to retreive the number of desired packets
	 * @param int $NumberOfPackets
	 * @return array
	 */
	public function ForceReceiveManagerPackets($NumberOfPackets = 1)
	{
		$manager_packets = $this->ReceiveManagerPackets();
		if ( count($manager_packets) == 0 )
		{
			return $this->ForceReceiveManagerPackets($NumberOfPackets);
		}
		$requested_packets = array();
		for ( $i = 0; $i < $NumberOfPackets; $i++ )
		{
			$requested_packets[] = $manager_packets[$i];
		}
		return $NumberOfPackets === 1 ? current($requested_packets) : $requested_packets;
	}
	
	/**
	 * parse the received packet from manager
	 * @param string
	 * @return array
	 * @access private
	 * @static
	 */
	private static function ParseManagerPacket($ManagerPacket)
	{
		$extra_data       = array();
		$parsed_packet    = array();
		$packet_lines     = explode(AmiConnector::MANAGER_ARG_DIV, $ManagerPacket);
		$privilege_regexp = "Privilege:[[:space:]a-zA-Z0-9]+" . AmiConnector::MANAGER_ARG_DIV;
		$no_privilege     = eregi($privilege_regexp, $ManagerPacket) === FALSE;
		/* get the response and actionid first */
		foreach ( $packet_lines as $packet_line )
		{
			$line_index = array_search($packet_line, $packet_lines); /* getting the real index */
			if ( strpos($packet_line, 'Event: ') !== FALSE ) /* event message? save it and delete it from packet lines */
			{
				$parsed_packet['Event'] = str_replace('Event: ', '', $packet_line);
				array_splice($packet_lines, $line_index, 1);
				break;
			}
			elseif ( strpos($packet_line, 'Response: ') !== FALSE ) /* response message? save it and delete it from packet lines */
			{
				$parsed_packet['Response'] = str_replace('Response: ', '', $packet_line);
				array_splice($packet_lines, $line_index, 1);
			}
			elseif ( strpos($packet_line, 'ActionID: ') !== FALSE )
			{
				$parsed_packet['ActionID'] = str_replace('ActionID: ', '', $packet_line);
				array_splice($packet_lines, $line_index, 1);
			}
			elseif ( strpos($packet_line, 'Privilege: ') !== FALSE )
			{
				$parsed_packet['Privilege'] = str_replace('Privilege: ', '', $packet_line);
				array_splice($packet_lines, $line_index, 1);
			}
			elseif ( ( $no_privilege || isset($parsed_packet['Privilege']) ) &&
			          isset($parsed_packet['Response']) && isset($parsed_packet['ActionID']) )
			{
				break;
			}
		}
		foreach ( $packet_lines as $packet_line )
		{
			/* if it is an event, or it is not a follows response packet */
			if ( ( array_key_exists('Event', $parsed_packet) ) ||
			         ( $parsed_packet['Response'] !== 'Follows' && strpos($packet_line, ': ') !== FALSE ) )
			{
				list($line_key, $line_value) = explode(': ', $packet_line);
				$parsed_packet[$line_key]    = $line_value;
			}
			else
			{
				$extra_data[] = trim($packet_line);
			}
		}
		if ( count($extra_data) > 0 )
		{
			$parsed_packet['ExtraData'] = $extra_data;
		}
		return $parsed_packet;
	}
	
	/**
	 * simple parser to receive the MAGI data
	 * @param string $MagiResult
	 * @return array
	 * @access public
	 */
	public static function ParseMagiExecResult($MagiResult)
	{
		/* since MAGI data is urlencoded, we need to decode it */
		$decoded_result = urldecode($MagiResult);
		$result_lines   = explode("\n", $decoded_result);
		$magi_result    = array();
		/* once decoded, then we need to separate the key: value pairs and feed the $magi_result array */
		foreach ( $result_lines as $result_line )
		{
			$trimmed_line = trim($result_line);
			if ( !empty($trimmed_line) )
			{
				list($key, $value) = explode(':', $trimmed_line);
				$magi_result[$key] = trim($value);
			}
		}
		return $magi_result;
	}

	public function Playback($Channel, $FileName)
	{
		$action = array
		(
			'Action'   => 'MAGI',
			'Channel'  => $Channel,
			'Command'  => 'EXEC Playback ' . $FileName
		);
		$this->commands_queue[] = $action;
	}
	
	public function Hangup($Channel)
	{
		$action = array
		(
			'Action'   => 'MAGI',
			'Channel'  => $Channel,
			'Command'  => 'EXEC Hangup'
		);
		$this->commands_queue[] = $action;
	}

	public function Dial($Channel, $DialString)
	{
		$action = array
		(
			'Action'   => 'MAGI',
			'Channel'  => $Channel,
			'Command'  => 'EXEC Dial ' . $DialString
		);
		$this->commands_queue[] = $action;
	}

	public function EnterVoiceMail($Channel, $VoiceBox)
	{
		$action = array
		(
			'Action'   => 'MAGI',
			'Channel'  => $Channel,
			'Command'  => 'EXEC VoicemailMain ' . $VoiceBox
		);
		$this->commands_queue[] = $action;
	}

	public function StartConference($Channel, $ConferenceNumber)
	{
		$action = array
		(
			'Action'   => 'MAGI',
			'Channel'  => $Channel,
			'Command'  => 'EXEC MeetMe ' . $ConferenceNumber
		);
		$this->commands_queue[] = $action;
	}

	public function CommitCommands()
	{
		foreach ( $this->commands_queue as $command )
		{
			$command['UniqueID'] = $command['Channel'];
			$this->SendDataToManager($command);
		}
		$this->commands_queue = array();
	}

	/**
	 * loggoff from manager
	 * @param void
	 * @return void
	 * @access private
	 */
	private function LogOff()
	{
		$this->SocketWrite("Action: logoff\r\n\r\n");
		$this->SocketClose();
		$this->connected = FALSE;
	}
}
?>
