<?php
/**
 * This file contains the SocketClient class
 * @package netsock
 */

/**
 * requires the file SocketLayer.php
 * that file contains the abstract root
 * class of the API
 */
require_once 'SocketLayer.php';

/**
 * SocketClient
 * this class help us to do basic socket stuff
 * This class depends on the class Logger, needs to be included by the implementator
 * @abstract
 */
abstract class SocketClient extends SocketLayer
{
	/**
	 * whether or not we are connected
	 * @var bool
	 * @access private
	 */
	private $connected              = FALSE;
	
	/**
	 * allowed to reconnect?
	 * @var bool
	 * @access private
	 */
	private $reconnect              = TRUE;
	
	/**
	 * how many times we will try to reconnect
	 * @var int
	 * @access private
	 */
	private $retry_limit            = 5;
	
	/**
	 * how many seconds we are going to wait to connect
	 * @var int
	 * @access private
	 */
	private $time_out               = 10;
	
	/**
	 * what to do when we successfully connect to the server
	 * @return void
	 * @access protected
	 */
	protected function OnConnect(){}
	
	/**
	 * what to do when we successfully reconnect to the server
	 * @return void
	 * @access protected
	 */
	protected function OnReconnect(){}
	
	/**
	 * what to do when we successfully disconnect from the server
	 * @return void
	 * @access protected
	 */
	protected function OnDisconnect(){}
	
	/**
	 * try to connect to the requested resource BaseSocketClient::$resource
	 * @param string $SocketUri
	 * @return void
	 * @access protected
	 * @throw SocketException, ConnectException, InvalidSocketResourceException
	 */
	protected function SocketConnect($SocketUri = NULL)
	{
		$this->socket_uri  = $SocketUri !== NULL ? $SocketUri : $this->socket_uri;
		list(, $socket_address, $socket_port) = SocketLayer::ParseSocketResource($this->socket_uri);
		$this->main_socket = SocketLayer::SocketCreate($this->socket_uri);
		/* Setting Socket to non-block, so we can handle time outs and connection retries */
		socket_set_nonblock($this->main_socket);
		$connect_time      = time();
		$retries           = 0;
		while ( @socket_connect($this->main_socket, $socket_address, $socket_port) === FALSE )
		{
			/* Getting socket status */
			$socket_status = socket_last_error($this->main_socket);
			/* If socket status is connecting or in progress we apply the time counter */
			if ( $socket_status == SocketLayer::SOCKET_STATUS_CONNECTING || $socket_status == SocketLayer::SOCKET_STATUS_IN_PROGRESS )
			{
				if ( ( time() - $connect_time ) >= $this->time_out )
        		{
        			socket_close($this->main_socket);
				}
				usleep(500);
			}
			/* If we are asked to reconnect after disconnect or unsuccessful connect, and connnection retry limit has not been reached then */
			elseif ( $this->reconnect && $this->retry_limit > $retries )
			{
				$retries++;
				Logger::WriteLog("Reconnection Attempt #$retries");
				sleep(1);
			}
			else
			{
				$socket_error = "socket_create() failed: " . socket_strerror($socket_status);
				Logger::WriteLog($socket_error, Logger::LOG_LEVEL_FATAL);
				throw new ConnectException($socket_error);
			}
			
		}
		socket_set_block($this->main_socket);
		$this->OnConnect();
		if ( $this->connected )
		{
			Logger::WriteLog('Connected successfully');
		}
		$this->connected = TRUE;
	}

	/**
	 * sets retry limit. how many times retry to connect
	 * @return void
	 * @param int $RetryLimit
	 * @access private
	 */
	public function SetRetryLimit($RetryLimit)
	{
		if ( 0 === $RetryLimit ) 
		{
			$this->reconnect = FALSE;
		}
		$this->retry_limit = (int)$RetryLimit;
	}

	/**
	 * just read the socket available info from the server
	 * this method will return the string read from the socket, or NULL in case of
	 * lost connection
	 * @param void
	 * @return string
	 * @access protected
	 */
	protected function SocketRead()
	{
		Logger::WriteLog("calling socket_read()", Logger::LOG_LEVEL_DEBUG);
		$data = socket_read($this->main_socket, SocketLayer::SOCKET_READ_BYTES);
		Logger::WriteLog("socket_read() returned something", Logger::LOG_LEVEL_DEBUG);
		if ( $data === FALSE || $data == "" )
		{
			if ( $this->reconnect )
			{
				$this->Reconnect();
			}
			$this->SocketPermanentDisconnect();
			return NULL;
		}
		Logger::WriteLog("the following data has been read from the server: {$data}", Logger::LOG_LEVEL_DEBUG);
		return $data;
	}
	
	/**
	 * returns a reference to the server socket
	 * @param void
	 * @return resource reference
	 * @access public
	 */
	public function &GetSocket()
	{
		return $this->main_socket;
	}
	
	/**
	 * just write to the socket available info to the server
	 * @param string $SocketData
	 * @return void
	 * @access protected
	 */
	protected function SocketWrite($SocketData)
	{
		Logger::WriteLog("Request for write to the server the following data: {$SocketData}", Logger::LOG_LEVEL_DEBUG);
		$written_bytes = socket_write($this->main_socket, $SocketData, strlen($SocketData));
		if ( $written_bytes !== FALSE )
		{
			Logger::WriteLog("Written {$written_bytes} bytes to the server", Logger::LOG_LEVEL_DEBUG);
		}
		elseif ( $this->reconnect && $this->Reconnect() )
		{
			$this->SocketWrite($SocketData);
		}
		Logger::WriteLog("exiting BaseSocketClient::SocketWrite()", Logger::LOG_LEVEL_DEBUG);
	}

	/**
	 * just write to the socket available info to the server
	 * @param void
	 * @return void
	 * @access private
	 */
	private function Reconnect()
	{
		$this->connected = FALSE;
		$this->SocketClose();
		Logger::WriteLog("attempting to reconnect...");
		$this->SocketConnect();
	}
	
	/**
	 * disconnect permanently from the server
	 * @param void
	 * @return void
	 * @access private
	 */
	private function SocketPermanentDisconnect()
	{
		$this->SocketClose();
		$this->reconnect = FALSE;
		$this->connected = FALSE;
		$this->OnDisconnect();
	}
	
	/**
	 * wrapper to close the socket
	 * @param void
	 * @return void
	 * @access protected
	 */
	protected function SocketClose()
	{
		socket_close($this->main_socket);
	}
}
?>
